############################################################################
#    Copyright (C) 2009 by Kushal Das                                      #
#    kushaldas@gmail.com                                                   #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

from django.core.management import setup_environ
import settings

setup_environ(settings)

from SonaDatabase.models import *
from SonaFilesystem.filesystem import *

import os
import glob
import urllib2
from PIL import Image, ImageQt

from PyQt4 import QtGui, QtCore, QtOpenGL
import PyQt4
from PyKDE4.kdecore import i18n
from FlickrUploaderUI import Ui_FlickrUploader
from UkhraFlickr import UkhraFlickr

class FlickrScene(QtGui.QGraphicsScene):
    """
    My own flickr scene
    """
    def __init__(self, parent=None):
        QtGui.QGraphicsScene.__init__(self)
        self.setBackgroundBrush(QtCore.Qt.lightGray)

class FlickrIcon(QtGui.QGraphicsPixmapItem):
    """This class shows evefry item in the scene"""
    def __init__(self, parent, path, orientation, x):
        QtGui.QGraphicsPixmapItem.__init__(self)
        self.parent = parent
        self.path = path
        self.x = x
        self.orientation = orientation
        self.setFlag(QtGui.QGraphicsItem.ItemIsSelectable)
        self.setFlag(QtGui.QGraphicsItem.ItemIsFocusable)
        self.setToolTip(os.path.basename(path))
        self.title = str(os.path.basename(path))
        self.description = ''
        self.tags = ''
        self.privacy = 0
        self.safety = 0

    def __eq__(self,other):
        return self.path == other.path

    def keyPressEvent(self, event):
        key = event.key()
        if key == 16777220:
            pass
        elif key == 16777223:
            self.parent.emit(QtCore.SIGNAL("removeFlickrIcons()"))

    def correctPos(self):
        self.setPos(self.x, 0)

    def focusInEvent(self, event):
        if self.parent.lastitem:
            self.parent.lastitem.gatherdata()
        self.parent.titleTxt.setText(self.title)
        self.parent.descTxt.setPlainText(self.description)
        self.parent.tagsTxt.setText(self.tags)
        if self.privacy != '':
            self.parent.privacyComboBox.setCurrentIndex(self.privacy)
        if self.safety != '':
            self.parent.safetyComboBox.setCurrentIndex(self.safety)

    def focusOutEvent(self, event):
        self.parent.lastitem = self

    def gatherdata(self):
        self.title = unicode(self.parent.titleTxt.text())
        self.description = unicode(self.parent.descTxt.toPlainText())
        self.tags = unicode(self.parent.tagsTxt.text())
        self.privacy = self.parent.privacyComboBox.currentIndex()
        self.safety = self.parent.safetyComboBox.currentIndex()

        print self.title, self.description, self.tags, self.privacy, self.safety

class FlickrUploader(QtGui.QDialog, Ui_FlickrUploader):
    def __init__(self, parent):
        QtGui.QDialog.__init__(self)
        self.setupUi(self)
        self.scene = FlickrScene(self)
        self.uploadView.setScene(self.scene)
        self.uploadView.setInteractive(True)
        #self.tabWidget.setTabEnabled(1,False)

        self.uploadView.setDragMode(1)

        self.images = []
        self.lastitem = None
        self.lastflickrdir = "."

        self.x = 0
        self.numberofimages = 0

        self.connect(self.addFileBttn,QtCore.SIGNAL("clicked()"),self.getfiles)
        self.connect(self.getTokenBttn,QtCore.SIGNAL("clicked()"),self.gettoken)
        self.connect(self,QtCore.SIGNAL("removeFlickrIcons()"), self.removeFlickrIcons)
        self.connect(self.uploadBttn,QtCore.SIGNAL("clicked()"), self.upload)

        try:
            self.flickr = UkhraFlickr()
            self.connect(self.flickr,QtCore.SIGNAL("updateProgressBar(PyQt_PyObject, PyQt_PyObject)"),self.updateFileProgressBar)
            self.connect(self.flickr, QtCore.SIGNAL("uploadDone()"), self.uploadDone)
            self.connect(self.flickr, QtCore.SIGNAL("removeFlickrIcon(PyQt_PyObject)"), self.removeFlickrIcon)
            self.connect(self.flickr, QtCore.SIGNAL("fileDone(PyQt_PyObject)"), self.fileDone)
        except:
            qm = QtGui.QErrorMessage(self)
            qm.showMessage(i18n("Can not connect to  flickr web service, check your net connection"))
            return
        self.tryAgain()


    def tryAgain(self):
        if not self.flickr.gettoken():
            self.disableBttns()
            self.flickr.openfirefox()
            print "No token"
        else:
            self.getTokenBttn.setEnabled(False)
            self.getSets()


    def getSets(self):
        sets = self.flickr.getSets()
        for photoset in sets:
            try:
                fset = FlickrPhotoSet.objects.filter(title=photoset.title)[0]
                #qb = QtCore.QByteArray.fromBase64(str(fset.thumbnail))
            except:
                #icon = photoset.primary
                #sizes = icon.getSizes()
                #data = None
                #for details in sizes:
                #    if details['label'] == 'Square':
                #        source = details['source']
                #        ulink = urllib2.urlopen(source)
                #        data = ulink.read()
                #        ulink.close()
                #        break
                #qb = QtCore.QByteArray(data)
                fset = FlickrPhotoSet()
                fset.set_id = photoset.id
                fset.title = photoset.title
                #fset.thumbnail = str(qb.toBase64())
                fset.save()
            #pixmap = QtGui.QPixmap()
            #pixmap.loadFromData(qb)
            #icon = QtGui.QIcon(pixmap)
            #self.setComboBox.addItem(icon,photoset.title, QtCore.QVariant(photoset.id))
            self.setComboBox.addItem(photoset.title, QtCore.QVariant(photoset.id))

    def gettoken(self):
        try:
            self.flickr.gettokenfromweb()
        except:
            print "You never authenticated"
            return
        self.enableBttns()
        self.getTokenBttn.setEnabled(False)
        self.getSets()

    def getfiles(self):
        files = QtGui.QFileDialog.getOpenFileNames(self,\
                         "Select one or more files to open",\
                         self.lastflickrdir,"Images (*.png *.jpg *.JPG)")
        self.addfiles(files)

    def addfiles(self,files):
        if len(files) != 0:
            self.lastflickrdir = os.path.dirname(str(files[0]))
        for disk_file_name in files:
            #try:
            disk_file_name = str(disk_file_name)
            dfile = DiskFile.objects.filter(name=disk_file_name)[0]
            tags = [fdb.tag.name for fdb in FileTags.objects.filter(file=dfile)]
            data = str(dfile.thumbnail)
            qb = QtCore.QByteArray.fromBase64(data)
            image = QtGui.QImage()
            image.loadFromData(qb,'JPEG')
            self.addFlickrIcon(image,disk_file_name, dfile.orientation, tags)
            #except:
            #    pass

    def removeFlickrIcons(self):
        selecteditems = self.scene.selectedItems()
        for flickricon in selecteditems:
            self.removeFlickrIcon(flickricon)


    def removeFlickrIcon(self, flickricon):
            self.scene.removeItem(flickricon)
            self.images.remove(flickricon)
            self.redrawScene(flickricon.x)

    def fileDone(self,flickricon):
        self.removeFlickrIcon(flickricon)

    def redrawScene(self, x):
        for flickricon in self.images:
            if flickricon.x > x:
                flickricon.x -= 200
                flickricon.correctPos()
                self.x -= 200

    def addFlickrIcon(self, image, disk_file_name, orientation, tags=[]):
        qp = QtGui.QPixmap.fromImage(image)
        flickricon = FlickrIcon(self, disk_file_name, orientation, self.x)
        tagline = ""
        for tag in tags:
            if tag.find(' ') != -1:
                tagline += ' "' + tag + '"'
            else:
                tagline += ' ' + tag
        flickricon.tags = tagline
        flickricon.setPixmap(qp)
        self.scene.addItem(flickricon)
        self.images.append(flickricon)
        flickricon.setPos(self.x, 0)
        self.x += 200

    def disableBttns(self):
        self.uploadBttn.setEnabled(False)
        self.fileprogressBar.setEnabled(False)
        self.totalprogressBar.setEnabled(False)
        self.addFileBttn.setEnabled(False)

    def enableBttns(self):
        self.uploadBttn.setEnabled(True)
        self.addFileBttn.setEnabled(True)

    def upload(self):
        if self.lastitem:
            self.lastitem.gatherdata()
        elif len(self.images) > 0:
            self.lastitem = self.images[0]
            self.lastitem.gatherdata()
        self.totalprogressBar.setValue(0)
        self.uploadView.setEnabled(False)
        self.uploadBttn.setEnabled(False)
        self.flickr.setImages(self.images)
        self.flickr.setGroupDetails(unicode(self.groupDescriptionTxt.toPlainText()),unicode(self.groupTagTxt.text()))
        self.numberofimages = len(self.images)
        self.flickr.start()

    def uploadDone(self):
        self.uploadView.setEnabled(True)
        self.totalprogressBar.setValue(100)
        self.numberofimages = 0
        self.enableBttns()


    def updateFileProgressBar(self,progress, done):
        if done:
            self.fileprogressBar.setValue(100)
            self.updateTotalProgressBar()
        else:
            self.fileprogressBar.setValue(int(progress))

    def updateTotalProgressBar(self):
        value = self.totalprogressBar.value()
        value += 100 / self.numberofimages
        self.totalprogressBar.setValue(value)
