############################################################################
#    Copyright (C) 2009 by Kushal Das                                      #
#    kushaldas@gmail.com                                                   #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

from django.core.management import setup_environ
import settings

setup_environ(settings)

from SonaDatabase.models import *
from SonaFilesystem.filesystem import *

import os
import sys
import glob
import shlex
from PIL import Image, ImageQt

import PyQt4
from PyQt4 import QtGui, QtCore, QtOpenGL
from PyKDE4.kdecore import ki18n, KAboutData, KCmdLineArgs
from PyKDE4.kdeui import *
from PyKDE4.kio import *
from PyKDE4.kdecore import KUrl, i18n


from gviewUI import Ui_MainWindow
from FlickrUploaderui import FlickrUploader
from exifui import ExifUI
from SideBar import SideBar
from Preferencesui import PreferencesDialog 


def checkfile(file):
    if file.endswith('.jpeg') or file.endswith('.jpg') or file.endswith('.JPG') or file.endswith('.nef'):
        return True
    else:
        return False

class FullScreen(QtGui.QGraphicsView):
    """For Full ScreenView"""
    def __init__(self):
        QtGui.QGraphicsView.__init__(self)
        self.path = None
        self.myscene = QtGui.QGraphicsScene(self)
        self.setScene(self.myscene)
        self.myscene.setBackgroundBrush(QtCore.Qt.black)
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.imagesize = None
        self.files = []
        self.setDragMode(QtGui.QGraphicsView.ScrollHandDrag)
        self.setFrameShadow(QtGui.QFrame.Plain)
        self.setFrameShape(QtGui.QFrame.NoFrame)

    def setFiles(self, files):
        """
        Files is a list if dictionary with
        'path','orientation' as keys
        """
        self.files = files

    def addFile(self, file):
        """
        Files is a dictionary with
        'path','orientation' as keys
        """
        self.files.append(file)

    def clearall(self):
        self.myscene.clear()
        self.files = []

    def showImage(self,path, imagesize=0):
        del self.myscene
        self.path = path
        self.myscene = QtGui.QGraphicsScene(self)
        self.setScene(self.myscene)
        self.myscene.setBackgroundBrush(QtCore.Qt.black)
        img = Image.open(path)
        qw = QtGui.QDesktopWidget()
        size = qw.availableGeometry().size()
        for file in self.files:
            if file['path'] == path:
                orientation = file['orientation']
        if orientation == 'Rotated 90 CCW':
            img = img.rotate(90)
        elif orientation == 'Rotated 90 CW':
            img = img.rotate(-90)
        if imagesize == 0:
            img.thumbnail((size.width(),size.height()), Image.ANTIALIAS) 
        image = ImageQt.ImageQt(img)
        pixmap = QtGui.QPixmap.fromImage(image)
        self.myscene.clear()
        pitem = QtGui.QGraphicsPixmapItem(pixmap)
        self.myscene.addItem(pitem)
        self.centerOn(pitem)

    def toggle(self):
        if self.isFullScreen():
            self.hide()
        else:
            self.show()
            self.showFullScreen()

    def keyPressEvent(self, event):
        key = event.key()
        if key == 70 or key == 16777274:
            self.toggle()
        elif key == 16777216 and self.isFullScreen():
            self.toggle()
        elif key == 49:
            self.showImage(self.path,1)
        elif key == 48:
            self.showImage(self.path,0)
        elif key == 16777235 or key == 16777238:
            for index, data in enumerate(self.files):
                if data['path'] == self.path:
                    if index != 0:
                        self.showImage(self.files[index - 1]['path'],0)
                        break
        elif key == 16777237 or key == 16777239:
            for index, data in enumerate(self.files):
                if data['path'] == self.path:
                    if index + 1 != len(self.files):
                        self.showImage(self.files[index + 1]['path'],0)
                        break

    def wheelEvent(self,event):
        if event.delta() < 0:
            for index, data in enumerate(self.files):
                if data['path'] == self.path:
                    if index + 1 != len(self.files):
                        self.showImage(self.files[index + 1]['path'],0)
                        break
        else:
            for index, data in enumerate(self.files):
                if data['path'] == self.path:
                    if index != 0:
                        self.showImage(self.files[index - 1]['path'],0)
                        break

class ImageIcon(QtGui.QGraphicsPixmapItem):
    """This class shows every item in the scene"""
    def __init__(self, parent, path, orientation):
        QtGui.QGraphicsPixmapItem.__init__(self)
        self.parent = parent
        self.path = path
        self.pixmap = None
        self.tags = []
        self.orientation = orientation
        self.setFlag(QtGui.QGraphicsItem.ItemIsSelectable)
        self.reloadTags()

    def reloadTags(self):
        dfiles = DiskFile.objects.filter(name=self.path)
        if len(dfiles) == 0:
            tip = os.path.basename(self.path)
            self.setToolTip(QtCore.QString(str(tip)))
            return
        dfile = dfiles[0]
        self.tags = []
        tags = FileTags.objects.filter(file=dfile)
        #tip = os.path.basename(self.path)
        tip = ''
        for dtag in tags:
            tip += "\n<b>%s</b>" % (dtag.tag.name)
            self.tags.append(dtag.tag.name)
        self.setToolTip(QtCore.QString(str(tip)))

    def __eq__(self,other):
        return self.path == other.path

    #def mouseDoubleClickEvent(self, event):
     #   """On mouse click select itself"""
        #self.label = FullScreen(self.path)
        #self.label.setPixmap(QtGui.QPixmap(self.path))
        #self.lafbel.showFullScreen()

    def mousePressEvent(self, event):
        self.parent.currentimage = self
        if  event.button() == 2:
            self.menu = KRun.displayOpenWithDialog(KUrl.List(KUrl(self.path)), self.parent)

    def setPixmapData(self, pixmap):
        self.pixmap = pixmap
        self.setPixmap(pixmap)

    #def paint(self,painter,option, widget):
    #    rect = option.rect
    #    thumbnailRect = QtCore.QRect(
    #        rect.left() + (rect.width() - self.pixmap.width())/2,
    #        rect.top() + (rect.height() - self.pixmap.height())/2 - 1,
    #        self.pixmap.width(),
    #       self.pixmap.height())
    #    painter.drawPixmap(thumbnailRect.left(), thumbnailRect.top(), self.pixmap)

class CollectionThread(QtCore.QThread):
    def __init__(self, files,disk, parent=None):
        QtCore.QThread.__init__(self,parent)
        self.allfiles = files
        self.stopsignals = False
        self.disk = disk

    def run(self):
        #FIXME!!!
        #Do some work please
        for num,disk_file_name in enumerate(self.allfiles):
            dfile = DiskFile.objects.filter(name=disk_file_name)
            if len(dfile) > 0:
                dfile = dfile[0]
                dfile.disk = self.disk
                dfile.save()
                print dfile.name, "is already in the db"
                if not self.stopsignals:
                    self.emit(QtCore.SIGNAL("updatecollectionbar(PyQt_PyObject)"),num)
                else:
                    return
                continue
            try:
                dfile = None
                exif = None
                #Get the EXIF data
                exifdata = exif_data(disk_file_name)
                dfile = DiskFile()
                dfile.name = disk_file_name
                dfile.path = os.path.dirname(disk_file_name)
                dfile.changetime = os.stat(disk_file_name)[8]
                exif = ExifData()

                orientation = exifdata.get('Exif.Image.Orientation')

                if orientation:
                    if orientation == '8':
                        orientation = 'Rotated 90 CCW'
                    elif orientation == '6':
                        orientation = 'Rotated 90 CW'
                    dfile.orientation = orientation

                img = Image.open(disk_file_name)

                if orientation == 'Rotated 90 CCW':
                    img = img.rotate(90)
                if orientation == 'Rotated 90 CW':
                    img = img.rotate(-90)

                img.thumbnail((160,120), Image.ANTIALIAS)

                i = ImageQt.ImageQt(img)


                bytearray = QtCore.QByteArray()
                qb = QtCore.QBuffer(bytearray)
                qb.open(PyQt4.QtCore.QIODevice.WriteOnly)
                i.save(qb,"JPEG")
                dfile.thumbnail = bytearray.toBase64().data()
                if not self.stopsignals:
                    self.emit(QtCore.SIGNAL("updatecollectionbar(PyQt_PyObject)"),num)
                else:
                    return
                dfile.disk = self.disk
                dfile.save()
                if exif:
                    exif.diskfile = dfile
                    exif.make = exifdata.get('Exif.Image.Make')
                    exif.model = exifdata.get('Exif.Image.Model')
                    exif.orientation = exifdata.get('Exif.Image.Orientation')
                    exif.exposure = exifdata.get('Exif.Photo.ExposureTime')
                    fnumber = exifdata.get('Exif.Photo.FNumber')
                    if fnumber:
                        fnumber = str("%.1f" % eval(fnumber+'.0'))
                    exif.fnumber = fnumber
                    exif.iso = exifdata.get('Exif.Photo.ISOSpeedRatings')
                    exif.exifversion = exifdata.get('Exif.Photo.ExifVersion')
                    exif.datetimeoriginal = exifdata.get('Exif.Photo.DateTimeOriginal')
                    exif.meteringmode = exifdata.get('Exif.Photo.MeteringMode')
                    exif.flash = exifdata.get('Exif.Photo.Flash')
                    focallength = exifdata.get('Exif.Photo.FocalLength')
                    if focallength:
                        focallength = str("%.1f" % eval(focallength+'.0'))
                    exif.focallength = focallength
                    exif.comment = exifdata.get('Exif.Photo.UserComment')
                    exif.save()

                #self.mutex.unlock()
            except Exception, e:
               print e.message



class ImageLoader(QtCore.QThread):

    def __init__(self, allfiles, parent=None):
        QtCore.QThread.__init__(self,parent)
        self.allfiles = allfiles
        self.stopsignals = False
        #self.mutex = mutex

    def run(self):
        #FIXME!!!
        #Do some work please
        for disk_file_name in self.allfiles:
            try:
                dfile = None
                exif = None
                #Get the EXIF data
                exifdata = exif_data(disk_file_name)
                try:
                    dfile = DiskFile.objects.filter(name=disk_file_name)[0]
                    dfile.changetime = os.stat(disk_file_name)[8]
                except:
                    dfile = DiskFile()
                    dfile.name = disk_file_name
                    dfile.path = os.path.dirname(disk_file_name)
                    dfile.changetime = os.stat(disk_file_name)[8]
                    exif = ExifData()



                orientation = exifdata.get('Exif.Image.Orientation')

                if orientation:
                    if orientation == '8':
                        orientation = 'Rotated 90 CCW'
                    elif orientation == '6':
                        orientation = 'Rotated 90 CW'
                    dfile.orientation = orientation

                img = Image.open(disk_file_name)

                if orientation == 'Rotated 90 CCW':
                    img = img.rotate(90)
                if orientation == 'Rotated 90 CW':
                    img = img.rotate(-90)

                img.thumbnail((160,120), Image.ANTIALIAS)

                i = ImageQt.ImageQt(img)


                bytearray = QtCore.QByteArray()
                qb = QtCore.QBuffer(bytearray)
                qb.open(PyQt4.QtCore.QIODevice.WriteOnly)
                i.save(qb,"JPEG")
                dfile.thumbnail = bytearray.toBase64().data()
                if not self.stopsignals:
                    self.emit(QtCore.SIGNAL("addImageIcon(PyQt_PyObject, PyQt_PyObject, PyQt_PyObject)"), i, disk_file_name, orientation)
                else:
                    return
                #self.mutex.lock()
                dfile.save()
                if exif:
                    exif.diskfile = dfile
                    exif.make = exifdata.get('Exif.Image.Make')
                    exif.model = exifdata.get('Exif.Image.Model')
                    exif.orientation = exifdata.get('Exif.Image.Orientation')
                    exif.exposure = exifdata.get('Exif.Photo.ExposureTime')
                    fnumber = exifdata.get('Exif.Photo.FNumber')
                    if fnumber:
                        fnumber = str("%.1f" % eval(fnumber+'.0'))
                    exif.fnumber = fnumber
                    exif.iso = exifdata.get('Exif.Photo.ISOSpeedRatings')
                    exif.exifversion = exifdata.get('Exif.Photo.ExifVersion')
                    exif.datetimeoriginal = exifdata.get('Exif.Photo.DateTimeOriginal')
                    exif.meteringmode = exifdata.get('Exif.Photo.MeteringMode')
                    exif.flash = exifdata.get('Exif.Photo.Flash')
                    focallength = exifdata.get('Exif.Photo.FocalLength')
                    if focallength:
                        focallength = str("%.1f" % eval(focallength+'.0'))
                    exif.focallength = focallength
                    exif.comment = exifdata.get('Exif.Photo.UserComment')
                    exif.save()

                #self.mutex.unlock()
            except Exception, e:
               print e.message


    #def __del__(self):
    #    self.wait()


class UkhraScene(QtGui.QGraphicsScene):
    """
    My own scene
    """
    def __init__(self, parent=None):
        QtGui.QGraphicsScene.__init__(self)
        self.parent = parent
        self.setBackgroundBrush(QtCore.Qt.gray)
        #self.menu = QtGui.QMenu("Open With")
        #self.gimpAction = self.menu.addAction("Gimp")
        #self.connect(self.gimpAction, QtCore.SIGNAL("triggered()"),self.openWithGimp)
        #self.item = None
        self.menu = None
        self.info = None
        self.special_keys = [16777222, 16777233, 16777237, 16777239, 16777234, 16777227, 16777236, 16777232, 16777235, 16777238]
        self.init()

    def init(self):
        self.x = 0
        self.y = 20
        self.row = 0
        self.column = 0

    def openWithGimp(self):
        print self.item.path

    def mouseDoubleClickEvent(self, event):
        item = self.itemAt(event.scenePos())
        if item:
            self.emit(QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), item, 0)

    def keyPressEvent(self, event):
        if not self.parent.currentimage:
            return
        key = event.key()
        x = self.parent.currentimage.x()
        y = self.parent.currentimage.y()
        item = None

        if key in self.special_keys:
            self.emit(QtCore.SIGNAL("doSpecialWork(PyQt_PyObject)"), key)
            return
        elif key == 85:
            self.emit(QtCore.SIGNAL("upload()"))
            return
        elif key == 16777220:
            #Enter key
            self.emit(QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), self.parent.currentimage, 0)
        elif key == 16777234:
            #Left Arrow key
            item = self.itemAt(x - 200,y)
            if item:
                path = QtGui.QPainterPath()
                path.addRect(x - 200, y, 20, 20)
                self.setSelectionArea(path)
        elif key == 16777236:
            #Right Arrow key
            item = self.itemAt(x + 200,y)
            if item:
                path = QtGui.QPainterPath()
                path.addRect(x + 200, y, 20, 20)
                self.setSelectionArea(path)
        elif key == 16777235:
            #Up Arrow key
            item = self.itemAt(x,y - 200)
            if item:
                path = QtGui.QPainterPath()
                path.addRect(x , y - 200, 20, 20)
                self.setSelectionArea(path)
        elif key == 16777237:
            #Down Arrow key
            item = self.itemAt(x,y + 200)
            if item:
                path = QtGui.QPainterPath()
                path.addRect(x , y + 200, 20, 20)
                self.setSelectionArea(path)
        elif key == 70 or key == 16777274:
            self.emit(QtCore.SIGNAL("viewFullScreenImage(PyQt_PyObject, PyQt_PyObject)"), self.parent.currentimage.path, 0)
            return

        if item:
            view = self.views()[0]
            view.centerOn(item)
            self.parent.currentimage = item

    def imageItems(self):
        items = []
        for x in self.items():
            if type(x) == ImageIcon:
                items.append(x)
        return items




class MainViewPort(QtGui.QGraphicsScene):
    """
    The view for viewing Images
    """
    def __init__(self, parent=None):
        QtGui.QGraphicsScene.__init__(self)
        self.parent = parent

    def keyPressEvent(self, event):
        key = event.key()
        item = self.items()[0]
        if key == 16777216:
            #If escape key then browse
            self.emit(QtCore.SIGNAL("browseImages()"))
        elif key == 16777234:
            #Left arrow key
            self.emit(QtCore.SIGNAL("previousImage()"))
        elif key == 16777236:
            #Right arrow key
            self.emit(QtCore.SIGNAL("nextImage()"))
        elif key == 49:
            #If 1 then show full size
            self.emit(QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), item, 1)
        elif key == 48:
            #If 0 then fit to screen
            self.emit(QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), item, 0)
        elif key == 70 or key == 16777274:
            self.emit(QtCore.SIGNAL("viewFullScreenImage(PyQt_PyObject, PyQt_PyObject)"), item.path, 0)


    def showfittoscreen(self):
        item = self.items()[0]
        self.emit(QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), item, 0)

class Viewer(QtGui.QGraphicsView):
    def __init__(self, parent):
        QtGui.QGraphicsView.__init__(self,parent)
        self.setRenderHint(QtGui.QPainter.Antialiasing, False)
        #Disabled for now
        #self.setDragMode(QtGui.QGraphicsView.ScrollHandDrag)

    def resizeEvent(self, event):
        pic_width = 200
        pic_height = 200
        scene = self.scene()
        if type(scene) == MainViewPort:
            scene.showfittoscreen()
            return
        if self.width() >= pic_width  * (scene.column + 1) or self.width() <= pic_width  * (scene.column - 1):
            pass
        else:
            return
        items = scene.items()
        scene.init()
        for imageicon in items:
            if type(imageicon) == QtGui.QGraphicsTextItem:
                scene.removeItem(imageicon)
                del imageicon
                continue
            columntest = scene.column + 1
            if pic_width * columntest < self.width():
                scene.x = pic_width * scene.column
                scene.column += 1
                scene.y = scene.row * pic_height
            else:
                scene.row += 1
                scene.column = 0
                scene.x = pic_width * scene.column
                scene.column += 1
                scene.y = scene.row * pic_height
            if imageicon.orientation == 'Rotated 90 CCW' or imageicon.orientation == 'Rotated 90 CW':
                imageicon.setPos( scene.x  ,scene.y )
            else:
                imageicon.setPos( scene.x ,scene.y )
            self.centerOn(imageicon)
            text = QtGui.QGraphicsTextItem(os.path.basename(imageicon.path))
            text.setDefaultTextColor(QtGui.QColor(255,255,255))
            scene.addItem(text)
            text.setPos(scene.x, scene.y + 120)



class gViewUI(Ui_MainWindow, KMainWindow):
    """Ui class"""
    def __init__(self):
        KMainWindow.__init__(self)
        self.setupUi(self)
        self.kmenu = self.helpMenu("", False)
        self.menubar.addMenu(self.kmenu)
        self.special_keys = [16777222, 16777233, 16777237, 16777239, 16777234, 16777227, 16777236, 16777232, 16777235, 16777238]

        del self.gView
        #Now setup my own view
        self.gView = Viewer(self.page)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.gView.sizePolicy().hasHeightForWidth())
        self.gView.setSizePolicy(sizePolicy)
        #self.gView.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        #self.gView.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.gView.setObjectName("gView")
        self.gridLayout.addWidget(self.gView, 0, 0, 1, 1)


        #self.gView = Viewer(self)
        #sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        #sizePolicy.setHorizontalStretch(0)
        #sizePolicy.setVerticalStretch(0)
        #sizePolicy.setHeightForWidth(self.gView.sizePolicy().hasHeightForWidth())
        #self.gView.setSizePolicy(sizePolicy)
        #self.gView.setObjectName("gView")
        #self.gridLayout.addWidget(self.gView, 0, 1, 1, 1)

        self.stack = QtGui.QStackedWidget(self)
        self.stack.hide()
        self.gridLayout.addWidget(self.stack, 0, 2, 1, 1)
        self.sidebar = SideBar(self, self.stack)
        self.gridLayout.addWidget(self.sidebar, 0, 3, 1, 1)

        self.initThumbnailScene()
        self.viewscene = MainViewPort(self)
        self.viewscene.setBackgroundBrush(QtCore.Qt.black)
        self.uploader = None
        self.ct = None

        self.searchscene = UkhraScene(self)
        self.fsc = FullScreen()
        #import pdb;pdb.set_trace()


        self.gView.setScene(self.qgscene)
        self.gView.setInteractive(True)
        self.files = []
        self.searchflag = False
        self.dir = None
        self.dir = QtCore.QDir().home()
        self.thread = None
        self.currentimage = None
        self.image_view_flag = False
        self.mutex_for_Image = QtCore.QMutex()

        self.toolBar.setIconSize(QtCore.QSize(32,32))
        self.preference = PreferencesDialog()
        self.preference.load()

        #For the statusbar
        self.totalimages = 0
        self.imagesloaded = 0

        for disk in Disk.objects.all():
            self.collectionList.addItem(disk.name)


        self.changeDir(self.dir.path())
        self.connectSlots()

    def connectSlots(self):
        """Connect signals to proper slots"""
        self.connect(self.addressList, QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"), self.gotoDirByList)
        self.connect(self.searchscene,QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), self.viewImage)
        self.connect(self.searchscene,QtCore.SIGNAL("viewFullScreenImage(PyQt_PyObject, PyQt_PyObject)"), self.viewFullScreenImage)
        self.connect(self.viewscene,QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), self.viewImage)
        self.connect(self.viewscene,QtCore.SIGNAL("browseImages()"), self.browseImages)
        self.connect(self.viewscene,QtCore.SIGNAL("previousImage()"), self.openPreviousImage)
        self.connect(self.viewscene,QtCore.SIGNAL("nextImage()"), self.openNextImage)
        self.connect(self.qgscene,QtCore.SIGNAL("upload()"), self.uploadtoFlickr)
        self.connect(self.actionOpen,QtCore.SIGNAL("triggered()"),self.openFolder)
        self.connect(self.actionOpen_Folder,QtCore.SIGNAL("triggered()"),self.openFolder)
        self.connect(self.action_Quit,QtCore.SIGNAL("triggered()"),self.queryClose)
        self.connect(self.actionBrowse,QtCore.SIGNAL("triggered()"),self.browseImages)
        self.connect(self.actionNext,QtCore.SIGNAL("triggered()"),self.openNextImage)
        self.connect(self.actionPrevious,QtCore.SIGNAL("triggered()"),self.openPreviousImage)
        self.connect(self.actionPreferences,QtCore.SIGNAL("triggered()"),self.showPreference)
        self.connect(self.actionUpload_to_Flickr,QtCore.SIGNAL("triggered()"),self.uploadtoFlickr)
        self.connect(self.addTagBttn,QtCore.SIGNAL("clicked()"),self.addTags)
        self.connect(self.deleteTagBttn,QtCore.SIGNAL("clicked()"),self.deleteTags)
        self.connect(self.searchBttn,QtCore.SIGNAL("clicked()"),self.search)
        self.connect(self.searchTxt,QtCore.SIGNAL("returnPressed()"),self.search)
        self.connect(self.maintoolbox,QtCore.SIGNAL("currentChanged(int)"),self.changeview)
        self.connect(self.selectCollectionBttn,QtCore.SIGNAL("clicked()"),self.selectCollection)
        self.connect(self.addCollectionBttn,QtCore.SIGNAL("clicked()"),self.addCollection)
        self.connect(self.searchCollectionBttn,QtCore.SIGNAL("clicked()"),self.searchCollection)
        self.connect(self.collectionList,QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"),self.showcollection)

    def queryClose(self):
        """
        We ahve to call sys.exit() or else PyKDE4 crashes , some nasty bug
        """
        self.hide()
        sys.exit(0)

    def showPreference(self):
        """
        Show the config dialog
        """
        self.preference.show()

    def initThumbnailScene(self):
        """
        Init the scene to show thumbnails
        """
        self.qgscene = UkhraScene(self)
        self.connect(self.qgscene,QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), self.viewImage)
        self.connect(self.qgscene,QtCore.SIGNAL("viewFullScreenImage(PyQt_PyObject, PyQt_PyObject)"), self.viewFullScreenImage)
        self.connect(self.qgscene,QtCore.SIGNAL("selectionChanged()"), self.selectionChanged)
        self.connect(self.qgscene,QtCore.SIGNAL("upload()"), self.uploadtoFlickr)
        self.connect(self.qgscene,QtCore.SIGNAL("doSpecialWork(PyQt_PyObject)"), self.doSpecialWork)

    def uploadtoFlickr(self):
        """
        Shows the upload to flickr GUI
        """
        if not self.uploader:
            self.uploader = FlickrUploader(self)
        elif not self.uploader.flickr.token:
            self.uploader.tryAgain()
        self.uploader.show()
        self.uploader.addfiles([item.path for item in self.qgscene.selectedItems()])

    def doSpecialWork(self, key):
        """
        Do the special user specific shortcuts
        """
        items = [os.path.basename(item.path) for item in self.qgscene.selectedItems()]
        self.preference.load()
        message = self.statusbar.currentMessage()
        os.chdir(str(self.dir.path()))

        if self.special_keys.index(key) != -1:
            for item in items:
                command = self.preference.data[str(self.special_keys.index(key))]
                if command:
                    if command.find('%f') != -1:
                        command = command.replace('%f',item) 
                    self.statusbar.showMessage(command)
                    os.system(command)

        self.statusbar.showMessage(message)



    def changeview(self, index):
        if index == 2:
            self.mainstack.setCurrentIndex(1)
        else:
            self.mainstack.setCurrentIndex(0)

    def selectCollection(self):
        """
        Select a path
        """
        dirname = QtGui.QFileDialog.getExistingDirectory(self,
                  i18n("Select Path of the disk"),os.path.expanduser("~"))
        if dirname:
            self.collectionpathTxt.setText(dirname)
            self.collectionNameTxt.setText(os.path.basename(unicode(dirname)))

    def addCollection(self):
        """
        Add a new collection
        """
        disk_name = unicode(self.collectionNameTxt.text())
        lst = Disk.objects.filter(name=disk_name)
        if len(lst) > 0:
            qm = QtGui.QMessageBox.information(self,i18n("Collection"),i18n("Collection already exists"))
            return
        #Save the disk first
        d= Disk()
        d.name = disk_name
        d.save()


        #Get the tags
        tags = [x for x in unicode(self.collectiontagTxt.text()).split(',')]
        for tag in tags:
            lst = Tags.objects.filter(tag_name=tag)
            if len(lst) > 0:
                continue
            tagdb = Tags()
            tagdb.tag_name = tag
            tagdb.save()
            dbt = DiskByTag()
            dbt.disk = d
            dbt.tag = tagdb
            dbt.save()

        #Now work on the files
        allfiles = walk(str(self.collectionpathTxt.text()))
        files = [file for file in allfiles if checkfile(file)]
        self.ct = CollectionThread(files, d)
        self.collectionProgressBar.setMaximum(len(files) -1 )
        self.connect(self.ct,QtCore.SIGNAL("updatecollectionbar(PyQt_PyObject)"), self.updatecollectionbar)

        self.collectionsbuttons(False)

        self.ct.start()

    def updatecollectionbar(self,num):
        self.collectionProgressBar.setValue(num)
        if num == self.collectionProgressBar.maximum():
            self.collectionsbuttons(True)
            self.collectionList.addItem(self.ct.disk.name)
            self.collectionpathTxt.setText('')
            self.collectiontagTxt.setText('')
            self.collectionNameTxt.setText('')


    def collectionsbuttons(self,value):
        self.collectionpathTxt.setEnabled(value)
        self.collectiontagTxt.setEnabled(value)
        self.collectionNameTxt.setEnabled(value)
        self.selectCollectionBttn.setEnabled(value)
        self.addCollectionBttn.setEnabled(value)

    def showcollection(self, item):
        """
        Show all items from a collection
        """
        for row in range(0,self.resultTable.rowCount()):
            self.resultTable.removeRow(0)
        disk_name = str(item.text())
        d = Disk.objects.filter(name=disk_name)[0]
        allfiles = DiskFile.objects.filter(disk=d)
        for file in allfiles:
            row = 1
            ed = ExifData.objects.filter(diskfile=file)[0]
            if file.thumbnail:
                qb = QtCore.QByteArray.fromBase64(str(file.thumbnail))
                qp = QtGui.QPixmap()
                qp.loadFromData(qb,'JPEG')
                qicon = QtGui.QIcon(qp)
                self.resultTable.insertRow(0)
                self.resultTable.setRowHeight(0,170)
                self.resultTable.setColumnWidth(0,160)
                self.resultTable.setItem(0,0,QtGui.QTableWidgetItem(qicon,''))
                self.resultTable.setItem(0,1,QtGui.QTableWidgetItem(d.name))
                self.resultTable.setItem(0,2,QtGui.QTableWidgetItem(os.path.basename(file.name)))
                self.resultTable.setItem(0,3,QtGui.QTableWidgetItem(file.path))
                row += 1

    def searchCollection(self):
        """
        Search for the collections
        """
        self.collectionList.clear()
        terms = unicode(self.searchCollectionTxt.text()).split(',')
        if terms[0] == u'':
            for disk in Disk.objects.all():
                self.collectionList.addItem(disk.name)

        results = []
        for term in terms:
            tag = Tags.objects.filter(tag_name=term)
            if len(tag ) == 0:
                continue
            disks = DiskByTag.objects.filter(tag = tag[0])
            print disks
            for disk in disks:
                if disk not in results:
                    self.collectionList.addItem(disk.disk.name)
                    results.append(disk.disk.name)

    def addTags(self):
        """
        Get the tags and attach to the selected files
        """
        tagstxt = str(self.addTagsTxt.text())
        tags = [tokens.strip('"') for tokens in shlex.shlex(tagstxt)]
        files = [item.path for item in self.qgscene.selectedItems()]
        for tag in tags:
            fromdb = PhotoTags.objects.filter(name=tag)
            if len(fromdb) > 0:
                #the tag is in db
                dbtag = fromdb[0]
            else:
                dbtag = PhotoTags()
                dbtag.name = tag
                dbtag.save()

            for filename in files:
                dfile = DiskFile.objects.filter(name=filename)[0]
                if len(FileTags.objects.filter(file=dfile, tag=dbtag)) > 0:
                    continue
                else:
                    ftag = FileTags()
                    ftag.file = dfile
                    ftag.tag = dbtag
                    ftag.save()
        self.addTagsTxt.clear()
        for item in self.qgscene.selectedItems():
            item.reloadTags()
        self.selectionChanged()

    def deleteTags(self):
        """
        Delete the currently selected tag from tag list
        """
        items = self.qgscene.selectedItems()
        if len(items) > 0:
            imageicon = items[0]
        else:
            return
        item = self.tagList.currentItem()
        tag = str(item.text())
        dbtag = PhotoTags.objects.filter(name=tag)[0]
        dfile = DiskFile.objects.filter(name=imageicon.path)[0]
        ftag = FileTags.objects.filter(file=dfile, tag=dbtag)[0]
        ftag.delete()
        item = self.tagList.takeItem(self.tagList.currentRow())
        del item
        imageicon.reloadTags()


    def search(self):
        searchtxt = str(self.searchTxt.text())
        tags = [tokens.strip('"') for tokens in shlex.shlex(searchtxt)]
        result = []
        self.searchscene.clear()
        self.gView.setScene(self.searchscene)
        for tag in tags:
            fromdb = PhotoTags.objects.filter(name=tag)
            if len(fromdb) > 0:
                #the tag is in db
                dbtag = fromdb[0]
                files = FileTags.objects.filter(tag=dbtag)
                for f in files:
                    if f.file.name not in result:
                        result.append(f.file.name)
                        data = str(f.file.thumbnail)
                        qb = QtCore.QByteArray.fromBase64(data)
                        image = QtGui.QImage()
                        image.loadFromData(qb,'JPEG')
                        self.searchflag = True
                        self.addImageIcon(image,f.file.name, f.file.orientation)
        self.searchflag = True



    def openPreviousImage(self):
        """
        Open the Previous image if available
        """
        items = self.qgscene.imageItems()
        for index, data in enumerate(items):
            if data == self.currentimage:
                if index != 0:
                    self.currentimage = items[index - 1]
                    self.viewImage(items[index - 1])
                    break

    def openNextImage(self):
        """
        Open the next image if available
        """
        items = self.qgscene.imageItems()
        for index, data in enumerate(items):
            if data == self.currentimage:
                if index +1 != len(items):
                    self.currentimage = items[index + 1]
                    self.viewImage(items[index+1])
                    break

    def selectionChanged(self):
        """
        Get the currently selected file and show the EXIF information
        Also show the tags
        """
        items = self.qgscene.selectedItems()
        if len(items) > 0:
            item = items[0]
        else:
            return
        #Show the tags in the tag widget
        self.tagList.clear()
        for tag in item.tags:
            icon = QtGui.QIcon()
            icon.addPixmap(QtGui.QPixmap(":/icons/tag.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
            listitem = QtGui.QListWidgetItem(icon,tag)
            self.tagList.addItem(listitem)
        try:
            file = DiskFile.objects.filter(name=item.path)[0]
            details = ExifData.objects.filter(diskfile=file)[0]
        except:
            #No Exif Info
            return
        result = {}
        result['make'] = details.make
        result['model'] = details.model
        result['orientation'] = details.orientation
        result['exposure'] = details.exposure
        result['focallength'] = details.focallength
        result['iso'] = details.iso
        result['date'] = details.datetimeoriginal
        result['metering'] = details.meteringmode
        result['aparture'] = details.fnumber
        result['flash'] = details.flash
        result['comments'] = details.comment
        self.sidebar.addDetails(result)

    def setDir(self,name):
        self.dir = QtCore.QDir(name)

    def openFolder(self):
        """
        It opens a folder selecting dialog
        """
        dirname = QtGui.QFileDialog.getExistingDirectory(self,
                  i18n("Open Directory"),os.path.expanduser("~"))
        if dirname:
            self.setDir(dirname)
            self.changeDir(self.dir.absolutePath())


    def gotoDirByList(self, item):
        """To change dir when a user clicks on browse box
        It also adds/removes dir name from the addressBox
        """
        if str(item.text()) == '.':
            self.searchflag = False
            self.browseImages()
            return
        if str(item.text()) == '..':
            nextdir = os.path.dirname(str(self.dir.absolutePath()))
        else:
            nextdir = os.path.join(str(self.dir.absolutePath()), str(item.text()))
        if not os.access(nextdir, os.R_OK | os.X_OK):
            qm = QtGui.QErrorMessage(self,"Access Error")
            qm.showMessage("You don't have access to %s" % nextdir)
            return
        self.changeDir(nextdir)

    def changeDir(self,nextdir):
        """
        Change the view to the new directory
        """
        self.dirlabel.setText(nextdir)
        self.totalimages = 0
        self.imagesloaded = 0
        self.statusbar.showMessage('0/0')
        self.dir = QtCore.QDir(nextdir)
        self.files = []
        self.getDirs()
        self.rework()

    def getDirs(self):
        """Get the dir names and show"""
        self.addressList.clear()
        for file_entry in self.dir.entryInfoList():
            if str(self.dir.absolutePath()) == '/':
                if str(file_entry.fileName()) in ['.','..']:
                    continue
            if file_entry.isDir():
                icon = QtGui.QIcon()
                icon.addPixmap(QtGui.QPixmap(":/icons/folder-blue.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
                listitem = QtGui.QListWidgetItem(icon,file_entry.fileName())
                self.addressList.addItem(listitem)


    def rework(self):
        """
        Get the file listing and show the thumbnails
        """
        self.initThumbnailScene()
        self.searchflag = False
        self.files = []
        if self.thread:
            self.thread.stopsignals = True
        #self.connect(self.qgscene,QtCore.SIGNAL("viewImage(PyQt_PyObject)"), self.viewImage)
        self.gView.setScene(self.qgscene)
        self.gView.setInteractive(True)
        size = self.gView.size()
        self.qgscene.init()
        self.qgscene.clear()

        width = size.width()
        pic_width = 200
        height = size.height()
        pic_height = 180
        currentfiles = os.listdir(self.dir.absolutePath())
        currentfiles.sort()
        for disk_file in currentfiles:
            number_of_images = glob.glob(os.path.join(str(self.dir.absolutePath()), "*.jpg"))
            number_of_images.extend(glob.glob(os.path.join(str(self.dir.absolutePath()), "*.JPG")))
            self.totalimages = len(number_of_images)
            if disk_file.endswith(".jpg") or disk_file.endswith(".JPG"):
                disk_file_name = os.path.join(str(self.dir.absolutePath()), disk_file)
                qp = None
                try:
                    fdb = DiskFile.objects.filter(name=disk_file_name)[0]
                    if int(fdb.changetime) == os.stat(disk_file_name)[8]:
                        data = str(fdb.thumbnail)
                        qb = QtCore.QByteArray.fromBase64(data)
                        image = QtGui.QImage()
                        image.loadFromData(qb,'JPEG')
                        self.addImageIcon(image,disk_file_name, fdb.orientation)
                    else:
                        print "time never matched"
                        self.files.append(disk_file_name)
                except:
                    #The thumbnail is not is not in DB
                    #So populate the DB
                    self.files.append(disk_file_name)
        if len(self.files) != 0:
            self.thread = ImageLoader(self.files)
            self.connect(self.thread,QtCore.SIGNAL("addImageIcon(PyQt_PyObject, PyQt_PyObject, PyQt_PyObject)"), self.addImageIcon)
            self.thread.start()

    def browseImages(self):
        """
        Show the Image broswer
        """
        self.setWindowTitle(i18n("Pony"))
        self.actionBrowse.setChecked(True)
        if self.searchflag:
            self.gView.setScene(self.searchscene)
        else:
            self.gView.setScene(self.qgscene)
        if not self.currentimage:
            return
        try:
            self.gView.centerOn(self.currentimage)
        except:
            pass
        finally:
            self.viewscene.clear()
            #self.currentimage = None

    def viewImage(self, imageitem, zoomlevel = 0):
        """
        This function will show an Image
        """
        #First check if the path exists or not
        if type(imageitem) == QtGui.QGraphicsTextItem:
            return
        if os.path.exists(imageitem.path):
            self.actionBrowse.setChecked(False)
            size = self.gView.size()
            item = ImageIcon(self, imageitem.path, imageitem.orientation)

            img = Image.open(imageitem.path)
            if imageitem.orientation == 'Rotated 90 CCW':
                img = img.rotate(90)
            elif imageitem.orientation == 'Rotated 90 CW':
                img = img.rotate(-90)

            if zoomlevel == 0:
                img.thumbnail((size.width()-5,size.height()-5), Image.ANTIALIAS)
            image = ImageQt.ImageQt(img)

            pixmap = QtGui.QPixmap.fromImage(image)
            item.setPixmap(pixmap)

            #Recreating the scene so that the image comes in middle
            self.viewscene = MainViewPort(self)
            self.connect(self.viewscene,QtCore.SIGNAL("browseImages()"), self.browseImages)
            self.connect(self.viewscene,QtCore.SIGNAL("viewImage(PyQt_PyObject, PyQt_PyObject)"), self.viewImage)
            self.connect(self.viewscene,QtCore.SIGNAL("previousImage()"), self.openPreviousImage)
            self.connect(self.viewscene,QtCore.SIGNAL("nextImage()"), self.openNextImage)
            self.connect(self.viewscene,QtCore.SIGNAL("viewFullScreenImage(PyQt_PyObject, PyQt_PyObject)"), self.viewFullScreenImage)
            self.viewscene.setBackgroundBrush(QtCore.Qt.black)

            #Add the item
            self.viewscene.addItem(item)
            self.gView.setScene(self.viewscene)
            self.setWindowTitle(os.path.basename(imageitem.path))
        else:
            print "File does not exists"

    def viewFullScreenImage(self, path, size):
        self.fsc = FullScreen()
        files = []
        for item in self.gView.scene().items():
            if type(item) == ImageIcon:
                files.append({'path':item.path,'orientation':item.orientation})
        self.fsc.setFiles(files)
        self.fsc.toggle()
        self.fsc.showImage(path,size)

    def addImageIcon(self,image, disk_file_name, orientation):
        self.mutex_for_Image.lock()

        #Update the statusbar
        self.imagesloaded += 1
        self.statusbar.showMessage(str(self.totalimages) + '/' + str(self.imagesloaded))

        qp = QtGui.QPixmap.fromImage(image)
        imageicon = ImageIcon(self, disk_file_name, orientation)
        #self.connect(imageicon,QtCore.SIGNAL("showImage(PyQt_PyObject)"), self.showImage)
        imageicon.setPixmapData(qp)

        pic_width = 200
        pic_height = 200
        #x = 0

        if self.searchflag:
            scene = self.searchscene
        else:
            scene = self.qgscene
        columntest = scene.column + 1
        if pic_width * columntest < self.gView.width():
            scene.x = pic_width * scene.column
            scene.column += 1
            scene.y = scene.row * pic_height
        else:
            scene.row += 1
            scene.column = 0
            scene.x = pic_width * scene.column
            scene.column += 1
            scene.y = scene.row * pic_height

        scene.addItem(imageicon)
        #imageicon.setPos( x + 150 * column - width * 4,y + 200 * row - height * 4)
        #self.gView.setSceneRect( x + 150 * column - width * 4,y + 200 * row - height * 4, width, height)
        if orientation == 'Rotated 90 CCW' or orientation == 'Rotated 90 CW':
            imageicon.setPos( scene.x ,scene.y )
        else:
            imageicon.setPos( scene.x ,scene.y )

        text = QtGui.QGraphicsTextItem(os.path.basename(disk_file_name))
        text.setDefaultTextColor(QtGui.QColor(255,255,255))
        scene.addItem(text)
        text.setPos(scene.x, scene.y + 120)
        if self.searchflag:
            self.searchflag = False
        self.mutex_for_Image.unlock()
        #self.gView.setSceneRect( x + 150 * column,y + 200 * row, width, height)



class gViewApp(KApplication):
    """The application class"""
    def __init__(self,args=None):
        appName     = "pony"
        catalog     = ""
        programName = ki18n ("Pony")
        version     = "0.3"
        description = ki18n ("Pony , an image manager")
        license     = KAboutData.License_GPL
        copyright   = ki18n ("(c) 2009 Kushal Das")
        text        = ki18n ("Kushal's image manager")
        homePage    = "www.fedorahosted.org/pony"
        bugEmail    = "kushal@fedoraproject.org"
        self.aboutData   = KAboutData (appName, catalog, programName, version, description,\
                      license, copyright, text, homePage, bugEmail)
        KCmdLineArgs.init (sys.argv, self.aboutData)
        KApplication.__init__(self)

        self.mywindow = gViewUI()
        self.mywindow.show()
        self.exec_()

